<?php

    class Foreach_Item_In_List_Action extends Reactor_Action
    {
        protected $tpl = 'foreach_item_in_list.php';
        protected $name = 'Run event chain x times / for each item';
		
		public static $registration = array(
			ACTION_NAME => 'Run event chain x times / for each item',
			ACTION_CATEGORY => 'Looping',
			ACTION_DESCRIPTION => 'Runs an event chain, allowing context variables to be passed.',
			ACTION_AUTHOR => 'Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '08/06/2014',
			ACTION_AUTHLEVEL => 0,
			ACTION_TYPE => 'core',
			ACTION_META => array()
		);
		
        protected $fields = array(
            'run_type' => array(
                'label' => 'run type',
                'validate' => 'trim|require'
            ),
            'run_times' => array(
                'label' => 'times to run',
                'validate' => 'trim|callback_check_run_times'
            ),
            'item_list' => array(
                'label' => 'list of items',
                'validate' => 'trim|callback_check_item_list'
            ),
            'item_delimiter' => array(
                'label' => 'list delimiter',
                'validate' => 'trim|callback_check_item_delimiter'
            ),
            'item_context_name' => array(
                'label' => 'item context name',
                'validate' => 'trim|callback_check_item_context_name'
            ),
            'run_endpoint' => array(
                'label' => 'endpoint',
                'validate' => 'trim|required'
            ),
            'identifier' => array(
                'label' => 'identifier',
                'validate' => 'xss_clean|callback_check_identifier'
            ),
            'run_url' => array(
                'label' => 'run url',
                'validate' => 'trim|xss_clean|callback_check_url',
            ),
            'api_key' => array(
                'label' => 'api key',
                'validate' => 'trim|xss_clean|callback_check_api_key',
            ),
            'run_with_context' => array(
                'label' => 'run_with_context',
                'validate' => 'trim',
            ),
            'task_type' => array(
                'label' => 'task type',
                'validate' => 'trim|xss_clean|callback_check_task_type'
            ),
        );

        public function get_label()
        {
            if ($params = $this->get_params()) {
				$run_endpoint = $params['run_endpoint'];
				$identifier = $params['identifier'];
				$task_type = $params['task_type'];
                $items = $params['item_list'];
                $run_type = $params['run_type'];

				$estr = "";
				if ($run_endpoint == "local") {
					$estr =" local";
                } else {
					$estr = "remote";
                }
				
				$tstr = "?";
				if ($task_type == "sync") {
					$tstr = "synchronously";
                } else {
					$tstr = "asynchronously";
                }

                if ($run_type == "times") {
                    $times = $params['run_times'];
                    return sprintf('Run %s event chain \'%s\' %s times', $estr, $identifier, $times);
                } else {
                    return sprintf('Run %s event chain \'%s\' %s for each item in: %s', $estr, $identifier, $tstr, $items);
                }
            }
            return parent::get_label();
        }		
		
        public function check_identifier($str)
        {

            $endpoint = $this->ci->input->post('run_endpoint');
            if ($endpoint == 'local') {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_identifier', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function check_task_type($str)
        {

            $endpoint = $this->ci->input->post('run_endpoint');
            if ($endpoint == 'local') {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_task_type', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }


        /** 
         * Form validation function to check a url
         **/
        public function check_url($str)
        {
            $endpoint = $this->ci->input->post('run_endpoint');
            if ($endpoint == 'remote') {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_url', _('This field is required.'));
                    return FALSE;
                }
            } else {
                return TRUE;
            }
            $url_expression = "/^(https?|ftp)\:\/\/([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?[a-z0-9+\$_-]+(\.[a-z0-9+\$_-]+)*(\:[0-9]{2,5})?(\/([a-z0-9+\$_-]\.?)+)*\/?(\?[a-z+&\$_.-][a-z0-9;:@\/&%=+\$_.-]*)?(#[a-z_.-][a-z0-9+\$_.-]*)?\$/i";
            if (!preg_match($url_expression, $str)) {
                $this->ci->form_validation->set_message('check_url', 'The url specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        public function check_api_key($str)
        {
            $endpoint = $this->ci->input->post('run_endpoint');
            if ($endpoint == 'remote') {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_api_key', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        function check_run_times($str)
        {
            $run_type = $this->ci->input->post('run_type');
            if ($run_type == "times") {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_run_times', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        function check_item_list($str)
        {
            $run_type = $this->ci->input->post('run_type');
            if ($run_type == "list") {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_item_list', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        function check_item_delimiter($str)
        {
            $run_type = $this->ci->input->post('run_type');
            if ($run_type == "list") {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_item_delimiter', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        function check_item_context_name($str)
        {
            $run_type = $this->ci->input->post('run_type');
            if ($run_type == "list") {
                if (!strlen($str)) {
                    $this->ci->form_validation->set_message('check_item_context_name', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function run($context)
        {
            $this->set_context($context);
            $original_local_context = $this->get_param('run_with_context');

            $item_list = $this->get_param('item_list');
            $item_name = $this->get_param('item_context_name');
            $delimiter = $this->get_param('item_delimiter');

            // Do some magic if we are doing an x times run type
            if ($this->get_param('run_type') == "times") {
                $times = $this->get_param('run_times');
                $list = array();
                for ($x = 0; $x < $times; $x++) {
                    $list[] = $x;
                }
                $new_list = implode(',', $list);
                $item_list = $new_list;
                $delimiter = ',';
            }
            $items = explode($delimiter, $item_list);

            $result = 0;
            foreach ($items as $item) {

                // Prepend item to context variables being sent
                if (!empty($item_name)) {
                    $local_context = $item_name.":".$item."\n".$original_local_context;
                }

                if ($this->get_param('run_endpoint') == 'remote') {

                    $this->log('info', _('Got request to run remote chain.'));

                    if(!in_array('curl', get_loaded_extensions())) {
                        $this->log('error', 'cURL is not installed and is required for this function.');
                        $result = 1;
                        break;
                    }
                    $api_url = $this->get_param('run_url');
                    $post_query = array();
                    $post_query_str = "";
                    $post_query['api_key'] = $this->get_param('api_key');
                    if (!empty($local_context)) {
                        if ($lines = explode("\n", $local_context)) {
                            foreach ($lines as $line) {
                                list($var, $val) = explode(":", $line);
                                $post_query[trim($var)] = trim($val);
                            }
                        }
                    }
                    foreach ($post_query as $name => $val) {
                        $post_query_str .= sprintf("%s=%s&", $name, urlencode($val));
                    }

                    $post_query_str = rtrim($post_query_str, "&");

                    $this->log('info', sprintf('Performing POST request to %s', $api_url));
                    list($http_result, $http_info) = do_curl_request($api_url, $post_query_str);
                    $this->log('info', 'HTTP status was <code>'.$http_info['http_code'].'</code>');
                    $this->log('info', 'HTTP Result was<br/> <textarea rows="5" style="width:80%">'.$http_result.'</textarea>');
                    $s = $http_info['http_code'];
                    $result = (($s >= 200 && $s < 300) ? 0 : 1);

                } elseif ($this->get_param('run_endpoint') ==  'local') {

                    $identifier = $this->get_param('identifier');

                    $this->log('info', _('Got request to run local chain with identifier: '). $identifier);

                    if(!$event_chain = $this->ci->event_chain_model->get($identifier)) {
                        $this->log('error', _('The event chain could not be found.'));
                        $result = 1;
                        break;
                    }

                    if(!$event_chain->active) {
                        $this->log('error', _('The chain is inactive. Refusing to run.'));
                        $result = 1;
                        break;
                    }

                    // let's not allow event chains to run themselves asynchronously
                    if ($event_chain->id == $this->get_run()->get_event_chain()->id) {
                        $this->log('error', _('Chains are not allowed to run themselves asynchronously.'));
                        $result = 1;
                        break;
                    }

                    // check for required context
                    $this->log('info', _('Checking for context to run this chain.'));
                    $local_context_arr = array();
                    if (!empty($local_context)) {
                        $local_context = trim($local_context);
                        $items = explode("\n", $local_context);
                        $items = array_filter($items, 'trim');
                        foreach ($items as $item) {
                            if (preg_match("/:/", $item)) {
                                list($key, $value) = explode(":", $item);
                                $local_context_arr[$key] = $value;
                            }
                        }
                    }

                    if ($req = $event_chain->get_required_context()) {
                        $this->log('info', _('Ensuring required context has been provided.'));    
                        foreach ($req as $item) {
                            // load up default values
                            if (count($item) == 2) {
                                if (!isset($local_context_arr[$item[0]])) {
                                    $local_context_arr[$item[0]] = $item[1];
                                }
                            }
                            if (!isset($local_context_arr[$item[0]])) {
                                $this->log('error', _('Missing required context variable: '). $item[0]);
                                $result = 1;
                                break;
                            }
                        }
                    }
                    $this->log('info', _('Creating a new run for this chain and dispatching.'));
                    $run = $event_chain->new_run($local_context_arr);
                    if ($this->get_param('task_type') == 'async') {
                        $job_type = Net_Gearman_Task::JOB_BACKGROUND;
                        $this->log('info', _('Executing task asynchronously: not waiting for result.'));
                    } else {
                        $this->log('info', _('Executing task synchronously: waiting for result.'));
                        $this->log('info', _('Note: if the job is stuck here, it might mean you need to start more Gearman workers.'));
                        $job_type = Net_Gearman_Task::JOB_NORMAL;
                    }
                    if ($run->dispatch($job_type)) {
                        if ($this->get_param('task_type') == 'async') {
                            $this->log('info', _('The request was queued successfully as run #').$run->id);
                        } else {
                            $this->log('info', _('The event chain was executed successfully - run #').$run->id);
                        }
                    } else {
                        $this->log('info', _('The request dispatch failed.'));
                    }
                }

            }
            // End running for each item

            $this->set_result($result);
            return $context;
        }
        
    }
